@@ -37,16 +37,14 @@ def add_cli_commands(state):
 				print('Error: LDAP groups are not consistent with roles in database')
 				sys.exit(1)
 
-@bp.before_request
-@login_required()
-def role_acl(): #pylint: disable=inconsistent-return-statements
-	if not role_acl_check():
-		flash(_('Access denied'))
-		return redirect(url_for('index'))
-
 def role_acl_check():
 	return request.user and request.user.is_in_group(current_app.config['ACL_ADMIN_GROUP'])
 
+@bp.before_request
+@login_required(role_acl_check)
+def role_acl():
+	pass
+
 @bp.route("/")
 @register_navbar(25, lazy_gettext('Roles'), icon='key', blueprint=bp, visible=role_acl_check)
 def index():
